{{/* vim: set filetype=mustache: */}}
{{/*
Expand the name of the chart.
*/}}
{{- define "stacklight.name" -}}
{{- default .Chart.Name .Values.nameOverride | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/*
Create a default fully qualified app name.
We truncate at 63 chars because some Kubernetes name fields are limited to this (by the DNS naming spec).
If release name contains chart name it will be used as a full name.
*/}}
{{- define "stacklight.fullname" -}}
{{- if .Values.fullnameOverride -}}
{{- .Values.fullnameOverride | trunc 63 | trimSuffix "-" -}}
{{- else -}}
{{- $name := default .Chart.Name .Values.nameOverride -}}
{{- if contains $name .Release.Name -}}
{{- .Release.Name | trunc 63 | trimSuffix "-" -}}
{{- else -}}
{{- printf "%s-%s" .Release.Name $name | trunc 63 | trimSuffix "-" -}}
{{- end -}}
{{- end -}}
{{- end -}}

{{/*
Create chart name and version as used by the chart label.
*/}}
{{- define "stacklight.chart" -}}
{{- printf "%s-%s" .Chart.Name .Chart.Version | replace "+" "_" | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/*
Common labels
*/}}
{{- define "stacklight.labels" -}}
app.kubernetes.io/name: {{ include "stacklight.name" . }}
helm.sh/chart: {{ include "stacklight.chart" . }}
app.kubernetes.io/instance: {{ .Release.Name }}
{{- if .Chart.AppVersion }}
app.kubernetes.io/version: {{ .Chart.AppVersion | quote }}
{{- end }}
app.kubernetes.io/managed-by: {{ .Release.Service }}
{{- end -}}

{{/*
Helm Controller high availability ConfigMap name for leader election lock
*/}}
{{- define "stacklight.helmControllerLeaderLock" -}}
stacklight-helm-controller-leader-election
{{- end -}}

{{/*
Resources section
*/}}
{{- define "stacklight.resources" -}}
{{- $resources := mergeOverwrite (index (index .inTplValues.resourcesPerClusterSize .service | default (dict .inTplValues.clusterSize dict)) .inTplValues.clusterSize) (index .inTplValues.resources .service) }}
{{ dict "resources" $resources | toYaml }}
{{- end -}}

{{- define "stacklight.nodeSelector" -}}
{{- $nodeSelector := dict }}
{{- $nodeSelector = mergeOverwrite $nodeSelector .inTplValues.nodeSelector.default (index .inTplValues.nodeSelector.component .service | default dict) }}
{{- if $nodeSelector }}
nodeSelector:
{{ toYaml $nodeSelector | indent 2 }}
{{- end }}
{{- end -}}

{{- define "stacklight.tolerations" -}}
{{ $tolerations := concat .inTplValues.tolerations.default (index .inTplValues.tolerations.component .service | default list ) | uniq }}
{{- if $tolerations }}
tolerations:
{{ toYaml $tolerations | indent 2 }}
{{- end }}
{{- end -}}

{{- define "stacklight.alertmanager.route" -}}
{{- $allMatchers := .matchers }}
{{- range $label, $value := .match }}
  {{- $allMatchers = append $allMatchers (printf "%s=\"%s\"" $label $value) }}
{{- end }}
{{- range $label, $value := .match_re }}
  {{- $allMatchers = append $allMatchers (printf "%s=~\"%s\"" $label $value) }}
{{- end }}
{{- $resultRoute := dict }}
{{- with uniq $allMatchers }}
matchers:
  {{- toYaml . | nindent 2 }}
{{- end }}
{{- with .routes }}
routes:
  {{- toYaml . | nindent 2 }}
{{- end }}
{{- end -}}

{{- define "stacklight.clusterId" -}}
{{- if not (empty .Values.clusterId) -}}
{{- printf "%s" .Values.clusterId -}}
{{- else if .Values.telemetry.telemeterServer.enabled -}}
{{- printf "%s" .Values.telemetry.telemeterServer.id -}}
{{- else if .Values.telemetry.telemeterClient.enabled -}}
{{- printf "%s" .Values.telemetry.telemeterClient.id -}}
{{- end -}}
{{- end -}}

{{- define "stacklight.commonChartParams" -}}
{{- $artifacts := .root.Files.Get "files/artifacts.yaml" | fromYaml }}
{{- $chartVersion := index (index $artifacts "chartsVersions") .chart }}
{{- $chart := index $chartVersion "chart" }}
{{- $version := index $chartVersion "version" }}
{{- dict "chart" $chart "version" $version "namespace" .root.Release.Namespace | toYaml }}
{{- end -}}

{{- define "stacklight.getImage" -}}
  {{- $artifacts := .root.Files.Get "files/artifacts.yaml" | fromYaml }}
  {{- $images := index $artifacts "dockerImagesVersions" }}
  {{- $image := index $images .image }}
  {{- $registry := .root.Values.global.dockerBaseUrl }}
  {{- $imgSplit := split ":" $image }}
  {{- $repo := list $registry $imgSplit._0 | join "/" }}
  {{- if .flat }}
    {{- list $registry $image | join "/" }}
  {{- else }}
    {{- dict "repository" $repo "tag" $imgSplit._1 | toYaml }}
  {{- end }}
{{- end -}}

{{- define "memUnitEnum" -}}
{{- dict "" 0 "k" 1 "m" 2 "g" 3 | toYaml }}
{{- end -}}

{{- define "normalizeMem" -}}
{{- $mem := index . "memory" | int64 }}
{{- if not (index . "hasI") }}
  {{- $unit := index . "unit" }}
  {{- $unitDict := include "memUnitEnum" . | fromYaml }}
  {{- range $_ := until (index $unitDict $unit | int) }}
    {{- $mem = mul $mem 1000 }}
    {{- $mem = div $mem 1024 }}
  {{- end }}
{{- end }}
{{- $mem }}
{{- end -}}

{{- define "memStringToMap" -}}
{{- $hasI := hasSuffix "i" . }}
{{- $mem := . | trimSuffix "i" | lower }}
{{- $unit := "" }}
{{- range $u := keys (omit (include "memUnitEnum" . | fromYaml) "") }}
  {{- if hasSuffix $u $mem }}
    {{- $unit = $u }}
  {{- end }}
{{- end }}
{{- $mem = $mem | trimSuffix $unit | int64 }}
{{- $memNorm := include "normalizeMem" (dict "memory" $mem "unit" $unit "hasI" $hasI) | int64 }}
{{- dict "memory" $memNorm "unit" $unit | toYaml }}
{{- end -}}

{{- define "memStringToMB" -}}
{{- $memMap := include "memStringToMap" . | fromYaml }}
{{- $mem := index $memMap "memory" }}
{{- $unit := index $memMap "unit" }}
{{- if eq $unit "g" }}
  {{- $mem = mul $mem 1024 }}
{{- end }}
{{- $mem }}
{{- end -}}

{{- define "stacklight.logLevel" -}}
{{- $key := index (index .root.Values.stacklightLogLevels "component" | default dict) .component | default (index .root.Values.stacklightLogLevels "default" | default "") -}}
{{- index .logLevels $key | default "" -}}
{{- end -}}

{{- define "stacklight.customAlerts" -}}
{{- $customAlerts := dict }}
{{- range $_, $alert := .Values.prometheusServer.customAlerts }}
  {{- $_ := set $customAlerts $alert.alert (omit $alert "alert") }}
{{- end }}
{{- $customAlerts | toYaml }}
{{- end -}}

{{- define "stacklight.alertsFlags" -}}
{{- $externalEpMonitoringEnabled := not (empty (and .Values.externalEndpointMonitoring.enabled .Values.externalEndpointMonitoring.domains)) }}
{{- $sslCertMonitoringEnabled := not (empty (and .Values.sslCertificateMonitoring.enabled .Values.sslCertificateMonitoring.domains)) }}
alertmanager-webhook-servicenow: {{ .Values.alertmanagerSimpleConfig.serviceNow.enabled }}
alertmanager: true
calico: {{ .Values.calico.enabled }}
cassandra: {{ .Values.tungstenFabricMonitoring.enabled }}
ceph: {{.Values.ceph.enabled }}
docker-swarm-logs: {{ and .Values.mke.enabled .Values.logging.enabled }}
docker-swarm: {{ .Values.mke.enabled }}
elasticsearch: {{ .Values.logging.enabled }}
etcd: true
external-endpoint: {{ or $sslCertMonitoringEnabled $externalEpMonitoringEnabled }}
fluentd: {{ .Values.logging.enabled }}
general: true
grafana: true
ironic: {{ .Values.ironic.enabled }}
kafka: {{ .Values.tungstenFabricMonitoring.enabled }}
kube-prometheus-node-alerting: true
kubernetes-apps: true
kubernetes-resources: true
kubernetes-storage: true
kubernetes-system: true
libvirt: {{ .Values.openstack.enabled }}
mariadb: {{ .Values.openstack.enabled }}
mcc-exporter: {{ .Values.kaasExporterEnabled }}
memcached: {{ .Values.openstack.enabled }}
mke: {{ .Values.mke.enabled }}
msr: {{ .Values.msrMonitoring.enabled }}
nginx: {{ .Values.nginxExporter.enabled }}
node-network: true
node-time: true
openstack-cinder: {{ .Values.openstack.enabled }}
openstack-general: {{ .Values.openstack.enabled }}
openstack-general-logging: {{ and .Values.openstack.enabled .Values.logging.enabled }}
openstack-ironic: {{ and .Values.openstack.enabled .Values.openstack.ironic.enabled }}
openstack-neutron: {{ and .Values.openstack.enabled (not .Values.tungstenFabricMonitoring.enabled) }}
openstack-nova: {{ .Values.openstack.enabled }}
postgresql-ha: {{ .Values.highAvailabilityEnabled }}
postgresql: true
prometheus: true
prometheus-msteams: {{ .Values.alertmanagerSimpleConfig.msteams.enabled }}
prometheus-relay: true
pushgateway: true
rabbitmq: {{ or .Values.openstack.enabled .Values.tungstenFabricMonitoring.enabled }}
rabbitmq-operator: {{ .Values.tungstenFabricMonitoring.enabled }}
redis: {{ .Values.tungstenFabricMonitoring.enabled }}
removed: true
sf-notifier: {{ .Values.alertmanagerSimpleConfig.salesForce.enabled }}
ssl-mcc: {{ or .Values.mccTLSCertificateMonitoringEnabled (and (not (empty .Values.kaasTLSCertificateMonitoringEnabled)) .Values.kaasTLSCertificateMonitoringEnabled) }}
{{- $externalFQDNs := or (not (empty .Values.openstack.externalFQDN)) .Values.openstack.externalFQDNs.enabled }}
ssl-openstack: {{ and .Values.openstack.enabled $externalFQDNs }}
ssl: {{ $sslCertMonitoringEnabled }}
stacklight: true
telegraf: {{ or .Values.openstack.enabled .Values.mke.enabled (eq .Values.provider "BareMetal") }}
telegraf-arping-check: {{ and .Values.openstack.enabled (not .Values.tungstenFabricMonitoring.enabled) .Values.openstack.arpingCheck.enabled }}
telegraf-smart: {{ eq .Values.provider "BareMetal" }}
telemeter-client: {{ .Values.telemetry.telemeterClient.enabled }}
telemeter-server: {{ .Values.telemetry.telemeterServer.enabled }}
tungsten-fabric: {{ .Values.tungstenFabricMonitoring.enabled }}
tungsten-fabric-logging: {{ and .Values.tungstenFabricMonitoring.enabled .Values.logging.enabled }}
watchdog: {{ .Values.prometheusServer.watchDogAlertEnabled }}
zookeeper: {{ .Values.tungstenFabricMonitoring.enabled }}
{{- end -}}

{{/*
Collect all alerts in single dict
- omit disabled alert groups
- perform overrides
*/}}
{{- define "stacklight.collectAlerts" -}}
{{- $root := . }}
{{- $alertsFlags := include "stacklight.alertsFlags" $root | fromYaml }}
{{- $customAlerts := include "stacklight.customAlerts" $root | fromYaml }}
{{- $resultAlerts := dict }}
{{- range $path, $bytes := .Files.Glob "files/alerts/*.yaml" }}
  {{- $alertfile :=  $root.Files.Get $path | fromYaml }}
  {{- range $group, $groupAlerts := index $alertfile "alerts" }}
    {{- if index $alertsFlags $group }}
      {{- $resultAlerts = merge $resultAlerts $groupAlerts }}
    {{- end }}
  {{- end }}
{{- end }}
{{- $resultAlerts = mergeOverwrite $resultAlerts $customAlerts }}
{{ $resultAlerts | toYaml }}
{{- end -}}

{{- define "stacklight.getEnabledAlerts" -}}
{{- $alertsSieved := dict }}
{{- range $alertName, $alertValues := include "stacklight.collectAlerts" . | fromYaml }}
  {{- if or (not (hasKey $alertValues "enabled")) (index $alertValues "enabled") }}
    {{- $alertConfig := unset (omit $alertValues "_hidden") "enabled" }}
    {{- $_ := set $alertsSieved $alertName $alertConfig }}
  {{- end }}
{{- end }}
{{ $alertsSieved | toYaml }}
{{- end -}}

{{- define "stacklight.getGroupedAlerts" -}}
{{- $alertsGrouped := dict }}
{{- range $alertName, $alertConfig := include "stacklight.getEnabledAlerts" . | fromYaml }}
  {{- $labels := index $alertConfig "labels" }}
  {{- $serviceName := index $labels "service" }}
  {{- if not (hasKey $alertsGrouped $serviceName) }}
    {{- $_ := set $alertsGrouped $serviceName dict }}
  {{- end }}
  {{- $_ := set (index $alertsGrouped $serviceName) $alertName $alertConfig }}
{{- end }}
{{ $alertsGrouped | toYaml }}
{{- end -}}

{{- define "stacklight.collectAllInhibitRules" -}}
{{- $root := . }}
{{- $allInhibitRules := list }}
{{- $resultAlerts := include "stacklight.collectAlerts" $root }}
{{- range $alertname, $alertvalues := $resultAlerts | fromYaml }}
  {{- if or (not (hasKey $alertvalues "enabled")) (index $alertvalues "enabled") }}
    {{- if hasKey $alertvalues "_hidden" }}
      {{- if hasKey $alertvalues._hidden "inhibits" }}
        {{- range $rule := $alertvalues._hidden.inhibits }}
          {{- $source_matchers := list (printf "%s=%s" "alertname" $alertname) }}
          {{- if hasKey $rule "source_matchers" }}
            {{- $source_matchers = concat $source_matchers $rule.source_matchers }}
          {{- end }}
          {{- $_ := set $rule "source_matchers" $source_matchers }}
          {{- $allInhibitRules = append $allInhibitRules $rule }}
        {{- end }}
      {{- end }}
    {{- end }}
  {{- end }}
{{- end }}
{{- $allInhibitRules | toYaml }}
{{- end -}}
